@@ -26,3 +26,8 @@ def parse_response(content):
         return json.loads(content.decode('utf-8'))
     else:
         return eval(content)
+
+def compat_args(query):
+    for (key, value) in query.items():
+        if isinstance(value, bool):
+            query[key] = str(value).lower()
