@@ -13,13 +13,8 @@ operate with a Solr server.
 >>> query_response = solr.search(**query)
 
 """
-try:
-    from requests import async
-except:
-    pass
-
 from .response import SolrResponse
-from .compat import urljoin, get_wt
+from .compat import urljoin, get_wt, compat_args
 from xml.sax.saxutils import escape
 
 import json
@@ -28,13 +23,18 @@ import requests
 class Solr(object):
     """Acts as an easy-to-use interface to Solr."""
 
-    def __init__(self, base_url='http://localhost:8080/solr/'):
+    def __init__(self, base_url='http://localhost:8080/solr/', auth=None):
         """ Initializes a Solr object. Solr URL is a needed parameter.
+
+        :param base_url: Url to solr index
+        :param auth: Described in requests documentation:
+                     http://docs.python-requests.org/en/latest/user/quickstart/#basic-authentication 
         """
         self.base_url = base_url
         # base_url must be end with /
         if self.base_url[-1] != '/':
             self.base_url += '/'
+        self.auth = auth
 
     def search(self, resource='select', **kwargs):
         """Queries Solr with the given kwargs and returns a SolrResponse
@@ -49,7 +49,8 @@ class Solr(object):
         """
         query = build_request(kwargs)
         
-        http_response = requests.get(urljoin(self.base_url, resource), params=query)
+        http_response = requests.get(urljoin(self.base_url, resource),
+                                     params=query, auth=self.auth)
 
         solr_response = SolrResponse(http_response)
         return solr_response
@@ -57,7 +58,7 @@ class Solr(object):
     def search_cursor(self, resource='select', **kwargs):
         """ """
         query = build_request(kwargs)
-        cursor = Cursor(urljoin(self.base_url, resource), query)
+        cursor = Cursor(urljoin(self.base_url, resource), query, self.auth)
 
         return cursor
     
@@ -71,18 +72,16 @@ class Solr(object):
         :param size:     Size of threadpool
         :param resource: Request dispatcher. 'select' by default.
         """
-        url = urljoin(self.base_url, resource)
-
-        queries = map(build_request, queries)
-
-        rs = []
         try:
-            rs = [async.get(url, params=query) for query in queries]
-        except NameError:
-            raise RuntimeError('Gevent is required for Solr.async_search.')
+            import grequests
+        except:
+            raise RuntimeError('grequests is required for Solr.async_search.')
 
-        solr_responses = [SolrResponse(http_response) for http_response in async.map(rs, size=size)]
-        return solr_responses
+        url = urljoin(self.base_url, resource)
+        queries = map(build_request, queries)
+        rs = (grequests.get(url, params=query) for query in queries)
+        responses = grequests.map(rs, size=size)
+        return [SolrResponse(http_response) for http_response in responses]
 
 
     def update(self, documents, input_type='json', commit=True):
@@ -179,7 +178,7 @@ class Solr(object):
     def ping(self):
         """ Ping call to solr server. """
         url = urljoin(self.base_url, 'admin/ping')
-        http_response = requests.get(url, params={'wt': 'json'})
+        http_response = requests.get(url, params={'wt': 'json'}, auth=self.auth)
         return SolrResponse(http_response)
 
     def is_up(self):
@@ -190,6 +189,12 @@ class Solr(object):
             return False
         return solr_response.status == 200 and solr_response.solr_status == 0
 
+    def schema(self):
+        return self._get_file('schema.xml')
+
+    def solrconfig(self):
+        return self._get_file('solrconfig.xml')
+
     def more_like_this(self, resource='mlt', text=None, **kwargs):
         """Implements convenient access to Solr MoreLikeThis functionality  
 
@@ -230,16 +235,15 @@ class Solr(object):
             kwargs['wt'] = get_wt()
             headers = {'Content-type': 'text/json'}
             http_response = requests.post(urljoin(self.base_url, resource), 
-                                    params=kwargs,
-                                    data=text,
-                                    headers=headers)
+                                          params=kwargs,
+                                          data=text,
+                                          headers=headers,
+                                          auth=self.auth)
             solr_response = SolrResponse(http_response)
             return solr_response
         else:
             return self.search(resource=resource, **kwargs)
 
-
-
     def _post_xml(self, xml):
         """ Sends the xml to Solr server.
 
@@ -247,9 +251,12 @@ class Solr(object):
         """
         url = urljoin(self.base_url, 'update')
         xml_data = xml.encode('utf-8')
+        headers = {
+            'Content-type': 'text/xml; charset=utf-8',
+            'Content-Length': "%s" % len(xml_data)
+        }
         http_response = requests.post(url, data=xml_data,
-                                 headers={'Content-type': 'text/xml; charset=utf-8',
-                                          'Content-Length': "%s" % len(xml_data)})
+                                      headers=headers, auth=self.auth)
         return http_response
 
     def _post_json(self, json_doc):
@@ -259,18 +266,32 @@ class Solr(object):
         """
         url = urljoin(self.base_url, 'update/json')
         json_data = json_doc.encode('utf-8')
+        headers = {
+            'Content-type': 'application/json; charset=utf-8',
+            'Content-Length': "%s" % len(json_data)
+        }
         http_response = requests.post(url, data=json_data,
-                                 headers={'Content-type': 'application/json; charset=utf-8',
-                                          'Content-Length': "%s" % len(json_data)})
+                                      headers=headers, auth=self.auth)
         return http_response
 
+    def _get_file(self, filename):
+        """Retrieves config files of the current index."""
+        url = urljoin(self.base_url, 'admin/file')
+        params = {
+            'contentType': 'text/xml;charset=utf-8',
+            'file' : filename
+        }
+        http_response = requests.get(url, params=params, auth=self.auth)
+        return http_response.content
+
 
 class Cursor(object):
     """ Implements the concept of cursor in relational databases """
-    def __init__(self, url, query):
+    def __init__(self, url, query, auth=None):
         """ Cursor initialization """
         self.url = url
         self.query = query
+        self.auth = auth
 
     def fetch(self, rows=None):
         """ Generator method that grabs all the documents in bulk sets of 
@@ -286,7 +307,7 @@ class Cursor(object):
 
         self.query['start'] = 0
 
-        http_response = requests.get(self.url, params=self.query)
+        http_response = requests.get(self.url, params=self.query, auth=self.auth)
         solr_response = SolrResponse(http_response)
 
         while len(solr_response.documents) == self.query['rows']:
@@ -294,7 +315,8 @@ class Cursor(object):
 
             self.query['start'] += self.query['rows']
 
-            http_response = requests.get(self.url, params=self.query)
+            http_response = requests.get(self.url, params=self.query,
+                                         auth=self.auth)
             solr_response = SolrResponse(http_response)
 
         yield solr_response
@@ -331,5 +353,6 @@ def _get_add_xml(array_of_hash, overwrite=True):
 def  build_request(query):
     """ Check solr query and put convenient format """
     assert 'q' in query
+    compat_args(query)
     query['wt'] = get_wt()
     return query
